"""Defines function for returning program options.

$LastChangedDate: 2009-11-26 13:40:05 -0800 (Thu, 26 Nov 2009) $
$Rev: 26 $

"""


import os
import sys
import time


def get_process_name():
     return 'Mencoder'


def get_max_output_duration_delta_percentage():
     return 3


def get_default_min_duration():
    return 1


def get_accepted_input_extensions():
    return ('avi', 'mpeg', 'mpg', 'ts', 'vob')


def get_python_log_extension():
    return 'mencoder_python_log'


def get_executable_name():
    executable_name = 'mencoder'
    if os.name.lower() == 'nt':
        executable_name += '.exe'

    return executable_name


def get_executable_path():
    executable_name = get_executable_name()
    if os.path.exists(executable_name):
        return executable_name

    executable_path = os.path.join(os.path.dirname(sys.argv[0]),
                                   executable_name)
    if os.path.exists(executable_path):
        return executable_path

    raise Exception('Could not locate "%s" in working or script' \
                    ' directory' % executable_name)


def check_process_return_code(return_code):
    """Returns True if executable returned successfully, False otherwise"""
    return return_code == 0


def add_input_and_output_filenames(options, input_filename, output_filename):
    return options + ['-o', output_filename, input_filename]


def get_output_extension_options(media_stats_dict):
    bitrate = media_stats_dict.get('ID_VIDEO_BITRATE', 0)

    print 'Getting options for bitrate "%d"' % bitrate

    # High def
    if bitrate > 15000000:
        return 'avi', \
            [
             # Copy audio from input
             '-oac', 'copy',

             # Use lavc for video
             '-ovc', 'lavc',
             '-lavcopts', 'vcodec=mpeg4:vbitrate=4800:v4mv',
#             '-lavcopts', 'vcodec=mpeg4:vbitrate=4800',
            ]

    # Low def 
    return 'avi', \
        [
         # Use lame for audio
         '-oac', 'mp3lame',
         '-lameopts', 'cbr:br=224',

         # Inverse telecine (so far the best) (24m 36.8s for 30m show)
         '-vf', 'pullup,softskip,pp=md',

#         # Inverse telecine (second best) (21m 6.17s for 30m show)
#         '-vf', 'pullup,softskip,pp=li',

         # Use lavc for video
         '-ovc', 'lavc',
         '-lavcopts', 'vcodec=mpeg4:vbitrate=2400:v4mv:mbd=2:trell',
#         '-lavcopts', 'vcodec=mpeg4:vbitrate=2400:v4mv:mbd=2:trell' \
#                      ':cmp=3:subcmp=3:autoaspect:vpass=1',
        ]


def print_to_stdout_default(process, buffer_size=2048):
    """Buffers output and throws at "\rXXX\r" when possible for verbose
    program output.

    """
    # Initialize COM in this thread
    pythoncom.CoInitialize()

    last_line = ''
    process_pid = process.pid
    while process.poll() is None and is_alive(process_pid):
        line = process.stdout.read(buffer_size)

        if not line:
            time.sleep(1)
            continue

        if last_line:
            line = '%s%s' % (last_line, line)

        # Print all but last line
        if '\n' in line:
            lines = line.split('\n')
            for line in lines[:-1]:
                sys.stdout.write('%s\n' % line)

            line = lines[-1]

        # If there are multiple '\r's in the line, print just the last one
        if '\r' in line:
            lines = line.split('\r')

            # Print first and last parts
            sys.stdout.write('%s\n' % lines[0])

            line = lines[-1]

        last_line = line

    line = process.stdout.read() or ''
    line = line.replace('\r', '\n')
    line = '%s%s' % (last_line, line)
    if line:
        sys.stdout.write(line)


def print_mencoder_to_stdout(process, is_alive_check_func, buffer_size=2048):
    """Buffers output and prints limited mencoder output.

    """
    pos_time = 5

    last_line = ''
    process_pid = process.pid
    last_pos_print_time = 0
    while is_alive_check_func():
        line = process.stdout.read(buffer_size)

        if not line:
            time.sleep(1)
            continue

        if last_line:
            line = '%s%s' % (last_line, line)

        # Split on \r or \n
        lines = line.replace('\n', '\r').split('\r')

        # Chop off last partial line so as not to write incomplete lines
        last_line, lines = lines[-1], lines[:-1]

        pos_line_written_this_loop = False
        for line in lines:
            if not line:
                continue

            # Ignore " duplicate frame" lines
            if ' dup' in line and ' duplicate frame' in line:
               continue

            # Only write pos lines every pos_time seconds and only one
            # per this loop
            if line.startswith('Pos:'):
                if not pos_line_written_this_loop:
                    t = time.time()
                    if (t - last_pos_print_time) >= pos_time:
                        sys.stdout.write('%s\n' % line)
                        pos_line_written_this_loop = True
                        last_pos_print_time = t

                continue

            # Print this line
            sys.stdout.write('%s\n' % line)

    line = process.stdout.read() or ''
    line = line.replace('\r', '\n')
    line = '%s%s' % (last_line, line)
    if line:
        sys.stdout.write(line)


print_to_stdout = print_mencoder_to_stdout
